@@ -16,5 +16,8 @@ unless /^[^#]/ !~ (gem = $F[0])
       $F[3..-1] = []
     end
   end
-  $_ = [gem.name, gem.version, uri, *$F[3..-1]].join(" ")
+  f = [gem.name, gem.version.to_s, uri, *$F[3..-1]]
+  $_.gsub!(/\S+\s*/) {|s| (f.shift || "").ljust(s.size)}
+  $_ = [$_, *f].join(" ") unless f.empty?
+  $_.rstrip!
 end
